<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Twig\\' => array($vendorDir . '/twig/twig/src'),
    'Symfony\\Polyfill\\Php72\\' => array($vendorDir . '/symfony/polyfill-php72'),
    'Symfony\\Polyfill\\Mbstring\\' => array($vendorDir . '/symfony/polyfill-mbstring'),
    'Symfony\\Polyfill\\Ctype\\' => array($vendorDir . '/symfony/polyfill-ctype'),
    'Symfony\\Component\\Yaml\\' => array($vendorDir . '/symfony/yaml'),
    'Sabre\\Xml\\' => array($vendorDir . '/sabre/xml/lib'),
    'Sabre\\VObject\\' => array($vendorDir . '/sabre/vobject/lib'),
    'Sabre\\Uri\\' => array($vendorDir . '/sabre/uri/lib'),
    'Sabre\\HTTP\\' => array($vendorDir . '/sabre/http/lib'),
    'Sabre\\Event\\' => array($vendorDir . '/sabre/event/lib'),
    'Sabre\\DAV\\' => array($vendorDir . '/sabre/dav/lib/DAV'),
    'Sabre\\DAVACL\\' => array($vendorDir . '/sabre/dav/lib/DAVACL'),
    'Sabre\\CardDAV\\' => array($vendorDir . '/sabre/dav/lib/CardDAV'),
    'Sabre\\CalDAV\\' => array($vendorDir . '/sabre/dav/lib/CalDAV'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/Psr/Log'),
);
