<?php return array(
    'root' => array(
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'reference' => '3204b1c9210916d533e1bff02b5d3248fb2ced3a',
        'name' => 'fruux/baikal',
        'dev' => false,
    ),
    'versions' => array(
        'fruux/baikal' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'reference' => '3204b1c9210916d533e1bff02b5d3248fb2ced3a',
            'dev_requirement' => false,
        ),
        'jeromeschneider/baikal' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'dev-master',
            ),
        ),
        'psr/log' => array(
            'pretty_version' => '1.1.4',
            'version' => '1.1.4.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'reference' => 'd49695b909c3b7628b6289db5479a1c204601f11',
            'dev_requirement' => false,
        ),
        'sabre/dav' => array(
            'pretty_version' => '4.3.1',
            'version' => '4.3.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/dav',
            'aliases' => array(),
            'reference' => '130abb7017f56e0d99b04eb94b041e000a8e9b39',
            'dev_requirement' => false,
        ),
        'sabre/event' => array(
            'pretty_version' => '5.1.4',
            'version' => '5.1.4.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/event',
            'aliases' => array(),
            'reference' => 'd7da22897125d34d7eddf7977758191c06a74497',
            'dev_requirement' => false,
        ),
        'sabre/http' => array(
            'pretty_version' => '5.1.3',
            'version' => '5.1.3.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/http',
            'aliases' => array(),
            'reference' => '315f592adfcba8aeb73c2fd64285205747acbbd7',
            'dev_requirement' => false,
        ),
        'sabre/uri' => array(
            'pretty_version' => '2.2.2',
            'version' => '2.2.2.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/uri',
            'aliases' => array(),
            'reference' => '7cb0f489578afad5006e85cd60f18ff33f2d440d',
            'dev_requirement' => false,
        ),
        'sabre/vobject' => array(
            'pretty_version' => '4.4.1',
            'version' => '4.4.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/vobject',
            'aliases' => array(),
            'reference' => '06feff370141fd3118609f808e86d9315864bf14',
            'dev_requirement' => false,
        ),
        'sabre/xml' => array(
            'pretty_version' => '2.2.5',
            'version' => '2.2.5.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabre/xml',
            'aliases' => array(),
            'reference' => 'a6af111850e7536d200d9637c34885cd3c77a86c',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.25.0',
            'version' => '1.25.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'reference' => '30885182c981ab175d4d034db0f6f469898070ab',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-mbstring' => array(
            'pretty_version' => 'v1.25.0',
            'version' => '1.25.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-mbstring',
            'aliases' => array(),
            'reference' => '0abb51d2f102e00a4eefcf46ba7fec406d245825',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php72' => array(
            'pretty_version' => 'v1.25.0',
            'version' => '1.25.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php72',
            'aliases' => array(),
            'reference' => '9a142215a36a3888e30d0a9eeea9766764e96976',
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v3.4.47',
            'version' => '3.4.47.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'reference' => '88289caa3c166321883f67fe5130188ebbb47094',
            'dev_requirement' => false,
        ),
        'twig/twig' => array(
            'pretty_version' => 'v2.14.12',
            'version' => '2.14.12.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../twig/twig',
            'aliases' => array(),
            'reference' => '19c898bda30c5edea573bbb9ee1235d8cf6956ed',
            'dev_requirement' => false,
        ),
    ),
);
